using System;
using System.Collections.Generic;
using System.Linq;
using Encompass.Exceptions;

namespace Encompass
{
    [AttributeUsage(AttributeTargets.Class)]
    public class Reads : Attribute
    {
        public readonly HashSet<Type> readTypes = new HashSet<Type>();

        public Reads(params Type[] readTypes)
        {
            foreach (var readType in readTypes)
            {
                var isMessage = readType.GetInterfaces().Contains(typeof(IMessage));
                var isComponent = readType.GetInterfaces().Contains(typeof(IComponent));

                if (!isMessage && !isComponent)
                {
                    throw new IllegalReadTypeException("{0} must be a Message or Component", readType.Name);
                }

                if (isComponent)
                {
                    this.readTypes.Add(typeof(ComponentMessage<>).MakeGenericType(readType));
                }
                else
                {
                    this.readTypes.Add(readType);
                }
            }
        }
    }
}
